import { ref } from 'vue';

export class IdDispenser {
  #maxId: number;
  #usedIdSet: Set<number>;
  #unUsedIdSet: Set<number>;
  constructor() {
    this.#maxId = 0;
    this.#usedIdSet = new Set<number>();
    this.#unUsedIdSet = new Set<number>();
  }
  getId(...identifiersToRemove: number[]) {
    let id: number;
    if (this.#unUsedIdSet.size) {
      [id] = this.#unUsedIdSet;
      this.#unUsedIdSet.delete(id);
    } else {
      id = this.#maxId++;
    }
    this.#usedIdSet.add(id);
    this.removeId(...identifiersToRemove);
    return id;
  }
  removeId(...identifiers: number[]) {
    for (const id of identifiers) {
      if (!this.#usedIdSet.delete(id)) {
        return;
      }
      this.#unUsedIdSet.add(id);
    }
  }
  reset() {
    this.#maxId = 0;
    this.#unUsedIdSet.clear();
    this.#usedIdSet.clear();
  }
}
export function timeout(delay: number = 0) {
  return new Promise<void>((resolve) => setTimeout(resolve, delay));
}
export function create2DArray<T>(height: number, width: number, cell: T | (() => T)): T[][] {
  return Array.from({ length: height }, () =>
    Array.from({ length: width }, () => (cell instanceof Function ? cell() : cell))
  );
}
export function get2DArrayItem<T>(
  grid: T[][],
  first: number,
  second: number,
  colFirst: boolean
): T {
  return colFirst ? grid[first][second] : grid[second][first];
}
export function useRefresher() {
  const dispenser = new IdDispenser();
  const key = ref(dispenser.getId());
  return {
    key,
    refresh() {
      key.value = dispenser.getId(key.value);
    }
  };
}
